# -*- coding: utf-8 -*-
from copy import deepcopy
from urllib.parse import quote
import json

import requests

from capesbusca import SearchResult, SearchError

SEARCH_URL = "https://catalogodeteses.capes.gov.br/catalogo-teses/rest/busca"
AGGREGATIONS_URL = SEARCH_URL + \
    "/agregacoes?termo={}&agregado={}&pagina={}&registrosPorPagina={}"


def search(search_query, full_aggregations=False):
    '''
        Realiza a requisição de busca

        Params:
            search_query (search_query.SearchQuery): Query de busca
            full_aggregations (bool): Flag para realizar a requisição da lista completa de agregações

        Returns:
            (searchresult.SearchResult): Resultado da busca
    '''

    response_json = {}

    try:
        response_json = requests.post(SEARCH_URL, data=search_query.json(),
                                      headers={'content-type': 'application/json'}).json()

        if full_aggregations:
            response_json['agregacoes'] = get_full_aggregations(
                search_query, response_json.get('agregacoes', []))

    except json.decoder.JSONDecodeError:
        raise SearchError("Error doing the search")
    except ConnectionError:
        raise SearchError("Connection error")

    return SearchResult(response_json)


def get_full_aggregations(search_query, initial_aggregations):
    '''
        Tentar pegar todos os agregados das agregações

        Params:
            search_query (SearchQuery): query de busca
            initial_aggregation (dict): valor inicial das agregações
    '''

    aggregations = deepcopy(initial_aggregations)

    for aggregation in aggregations:
        aggregation_name = aggregation.get('campo', '') + '_raw'

        url = AGGREGATIONS_URL.format(
            quote(search_query.term), quote(aggregation_name), search_query.page, search_query.page_size)

        try:
            response = requests.post(url, data=search_query.json(), headers={
                'content-type': 'application/json'}, timeout=5).json()

            if response.get('total', 0) > 0:
                aggregation['agregados'] = response['agregados']

        except requests.exceptions.ReadTimeout:
            pass
        except json.decoder.JSONDecodeError:
            pass
        except ConnectionError:
            pass

    return aggregations



# -*- coding: utf-8 -*-


class Work:

    ''' Modelo dos trabalhos de tese e dissertação '''

    def __init__(self,
                 id='',
                 instituicao='',
                 nome_programa='',
                 municipio_programa='',
                 titulo='',
                 autor='',
                 data_defesa='',
                 volumes=0,
                 paginas=0,
                 biblioteca='',
                 grau_academico='',
                 link=''
                 ):
        '''
            Construtor da classe Work

            Attr:
                id (str): id do trabalho
                instituicao (str): instituição atribuída ao trabalho
                nome_programa (str): nome do programa atribuído ao trabalho
                municipio_programa (str): município do programa
                titulo (str): título do trabalho
                autor (str): autor do trabalho
                data_defesa (str): data da defesa do trabalho
                volumes (int): volumes do trabalho
                paginas (int): quantidade de páginas
                biblioteca (str): biblioteca que possuí o trabalho
                grau_academico (str): grau acadêmico do trabalho
                link (str): link de acesso do trabalho
        '''

        self.id = id
        self.instituicao = instituicao
        self.nome_programa = nome_programa
        self.municipio_programa = municipio_programa
        self.titulo = titulo
        self.autor = autor
        self.data_defesa = data_defesa
        self.volumes = volumes
        self.paginas = paginas
        self.biblioteca = biblioteca
        self.grau_academico = grau_academico
        self.link = link

    def __repr__(self):
        return str(vars(self))

    def __str__(self):
        return str(vars(self))

    @staticmethod
    def parse(dictionary):
        '''
            Reliza o parser de dados em dict para o objeto

            Params:
                dictionary (dict): dicionário com os dados

            Returns:
                Work
        '''

        id = dictionary.get('id', '')
        instituicao = dictionary.get('instituicao', '')
        nome_programa = dictionary.get('nomePrograma', '')
        municipio_programa = dictionary.get(
            'municipioPrograma', '')
        titulo = dictionary.get('titulo', '')
        autor = dictionary.get('autor', '')
        data_defesa = dictionary.get('dataDefesa', '')
        volumes = dictionary.get('volumes', 0)
        paginas = dictionary.get('paginas', 0)
        biblioteca = dictionary.get('biblioteca', '')
        grau_academico = dictionary.get('grauAcademico', '')
        link = dictionary.get('link', '')

        return Work(
            id=id,
            instituicao=instituicao,
            nome_programa=nome_programa,
            municipio_programa=municipio_programa,
            titulo=titulo,
            autor=autor,
            data_defesa=data_defesa,
            volumes=volumes,
            paginas=paginas,
            biblioteca=biblioteca,
            grau_academico=grau_academico,
            link=link,
        )



# -*- coding: utf-8 -*-
from json import dumps


class SearchQuery:

    ''' Classe para construir a query de busca '''

    def __init__(self):
        '''
            Construtor da classe SearchQuery

            Attr:
                term (str): termo da busca
                filters (array): filtros da busca
                page (int): página
                page_size (int): tamanho da página
        '''

        self.term = ""
        self.filters = []
        self.page = 1
        self.page_size = 20

    def json(self):
        '''	
            Cria o JSON da busca	
        '''

        return dumps({"termo": self.term,
                      "filtros": self.filters,
                      "pagina": self.page,
                      "registrosPorPagina": self.page_size})

    class Builder:

        ''' Classe para a construção da query de busca '''

        def __init__(self, term):
            '''
                Construtor da classe Builder

                Params:
                    term (str): termo da busca
            '''

            self.__search_query = SearchQuery()
            self.__search_query.term = term

        def build(self):
            '''
                Finaliza construção do objeto

                Returns:
                    (SearchResult): resultado da busca
            '''
            return self.__search_query

        def set_page(self, page):
            '''
                Define a página da busca

                Params:
                    page (int): número da página

                Returns:
                    (Builder): contrutor do SearchResult
            '''
            self.__search_query.page = page
            return self

        def set_page_size(self, page_size):
            '''
                Define o tamanho da página

                Params:
                    page_size (int): tamanho da página

                Returns:
                    (Builder): contrutor do SearchResult
            '''

            self.__search_query.page_size = page_size
            return self

        def add_filters(self, *args):
            '''
                Adiciona os filtros de busca

                Returns:
                    (Builder): contrutor do SearchResult
            '''

            for arg in args:
                self.__search_query.filters.append(vars(arg))

            return self



# -*- coding: utf-8 -*-
from capesbusca.filters import Filter


class Ano(Filter):

    ''' Filtro de ano '''

    def __init__(self, value):
        super().__init__(value)

        self.campo = "Ano"



# -*- coding: utf-8 -*-
from capesbusca.models.aggregate import Aggregate


class Aggregation:

    ''' Modelo para as agregações '''

    def __init__(self, campo='', total=0, agregados=[]):
        '''
            Construtor da classe Aggregation

            Attr:
                campo (str): nome da agregação
                total (int): total de agregados
                agregados (list): lista de agregados
        '''

        self.campo = campo
        self.total = total
        self.agregados = agregados

    def __repr__(self):
        return str(vars(self))

    def __str__(self):
        return str(vars(self))

    @staticmethod
    def parse(dictionary):
        '''
            Reliza o parser de dados em dict para o objeto

            Params:
                dictionary (dict): dicionário com os dados

            Returns:
                Aggregation
        '''

        campo = dictionary.get('campo', "")
        total = dictionary.get('total', "")
        agregados = []

        for agregado in dictionary.get('agregados', []):
            agregados.append(Aggregate().parse(agregado))

        return Aggregation(campo, total, agregados)



# -*- coding: utf-8 -*-
from capesbusca.models import Work, Aggregation


class SearchResult:

    ''' 
        Classe para receber a reposta da requisição 
        e realizar o parser para os modelos
    '''

    def __init__(self, result_json):
        '''
            Construtor da classe Response

            Attr:
                result_json (dict): resposta da requisição
                page (int): número da página
                page_size (int): tamanho da página
                total (int): número total de trabalhos do resultado da requisição
                works (list): lista de trabalhos
                aggregations (list): lista de agregações
        '''
        self.result_json = result_json
        self.page = 0
        self.page_size = 0
        self.total = 0
        self.works = []
        self.aggregations = []

        self.__parse_json()

    def __repr__(self):
        data_dict = vars(self)
        data_dict.pop('result_json')

        return str(vars(self))

    def __str__(self):
        data_dict = vars(self)
        data_dict.pop('result_json')

        return str(vars(self))

    def __parse_json(self):
        '''
            Realiza o parse da resposta da requisição para os modelos
        '''

        self.page = self.result_json.get('pagina', self.page)
        self.page_size = self.result_json.get(
            'registrosPorPagina', self.page_size)
        self.total = self.result_json.get('total', self.total)

        for work in self.result_json.get('tesesDissertacoes', self.works):
            self.works.append(Work().parse(work))

        for aggregation in self.result_json.get('agregacoes', self.aggregations):
            self.aggregations.append(Aggregation.parse(aggregation))



from capesbusca.filters.filter import Filter
from capesbusca.filters.ano import Ano
from capesbusca.filters.areaavaliacao import AreaAvaliacao
from capesbusca.filters.areaconcentracao import AreaConcentracao
from capesbusca.filters.areaconhecimento import AreaConhecimento
from capesbusca.filters.autor import Autor
from capesbusca.filters.banca import Banca
from capesbusca.filters.biblioteca import Biblioteca
from capesbusca.filters.grauacademico import GrauAcademico
from capesbusca.filters.instituicao import Instituicao
from capesbusca.filters.orientador import Orientador
from capesbusca.filters.programa import Programa



# -*- coding: utf-8 -*-
from capesbusca.filters import Filter


class AreaConhecimento(Filter):

    ''' Filtro da Área de Conhecimento '''

    def __init__(self, value):
        super().__init__(value)

        self.campo = "Área Conhecimento"




from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='capesbusca',
    packages=find_packages(),
    version='0.1.1',
    license='MIT',
    description='Biblioteca para realizar busca de termos no catálogo de teses e dissertações da Capes',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Vinícius Veríssimo',
    author_email='vinicius.matheus252@gmail.com',
    url='https://github.com/Vnicius/capes-busca',
    download_url='https://github.com/Vnicius/capes-busca/archive/v0.1.1.tar.gz',
    keywords=['data', 'api', 'capes', 'works'],
    install_requires=[
        'requests',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)



# -*- coding: utf-8 -*-


class Aggregate:

    ''' Modelo para os agregados '''

    def __init__(self, valor='', total=0):
        '''
            Construtor da classe Aggregate

            Attr:
                valor (str): valor do agregado
                total (int): total de trabalhos desse agregado
        '''

        self.valor = valor
        self.total = total

    def __repr__(self):
        return str(vars(self))

    def __str__(self):
        return str(vars(self))

    @staticmethod
    def parse(dictionary):
        '''
            Reliza o parser de dados em dict para o objeto

            Params:
                dictionary (dict): dicionário com os dados

            Returns:
                Aggregate
        '''

        valor = dictionary.get('valor', 0)
        total = dictionary.get('total', 0)

        return Aggregate(valor, total)
